/* ascii-numeric.cc - ascii-numeric comparator implementation
 * Rob Earhart
 * $Id: ascii-numeric.cc,v 1.2 1997/12/14 23:17:29 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "compare.h"
#include "init.h"

static int OrdAsciiNumeric(const String &first, const String &second) {
  String::const_iterator f = first.begin(), fe,
                         s = second.begin(), se;
  // First we make sure there's at least one 0-9 digit in each...
  if (first.end() - f == 0 || (*f < '0' && '9' < *f))
    if (second.end() - s == 0 || (*s < '0' && '9' < *s))
      return 0;
    else
      return -1;
  else if (second.end() - s == 0 || (*s < '0' && '9' < *s))
    return 1;

  // Next, we find the end of each string (in terms of 0-9 digits)
  for (fe = f; fe < first.end(); ++fe)
    if (*fe < '0' || '9' < *fe)
      break;

  for (se = s; se < second.end(); ++se)
    if (*se < '0' || '9' < *se)
      break;

  // Then we strip leading zeros off each until they're the same length
  while(se - s < fe - f)
    if (*f != '0')
      return *f - '0';
    else
      ++f;

  while(fe - f < se - s)
    if (*s != '0')
      return '0' - *s;
    else
      ++s;
  
  // Then we do a strict octet comparison
  while(fe - f)
    if (*f != *s)
      return *f - *s;
    else {
      ++f;
      ++s;
    }

  // And if we actually get here, the strings are considered equal.
  return 0;
}

static int OrdAsciiNumericReverse(const String &first, const String &second) {
  return -OrdAsciiNumeric(first, second);
}

static bool EqAsciiNumeric(const String &a, const String &b) {
  return (OrdAsciiNumeric(a, b) == 0);
}

static void SetupAsciiNumeric() {
  Compare::Family *family = Compare::Instance()->FindFamily("i");
  family->Orderings()["i;ascii-numeric"] = &OrdAsciiNumeric;
  family->Orderings()["+i;ascii-numeric"] = &OrdAsciiNumeric;
  family->Orderings()["-i;ascii-numeric"] = &OrdAsciiNumericReverse;
  family->Equalities()["i;ascii-numeric"] = &EqAsciiNumeric;
  family->Comparators().insert("i;ascii-numeric");
}

static Init init(&SetupAsciiNumeric);
