/* clientstr.cc - clientstr implementation
 * Rob Earhart
 * $Id: clientstr.cc,v 1.3 1998/03/09 00:57:51 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include <stdarg.h>
#include "clientstr.h"

bool ClientStr::contains_unsafe_chars(const String *str) {
  for (String::const_iterator it = str->begin();
       it != str->end();
       ++it)
    // This really should allow a string that's utf-8...
    if (*it == 0x00
	|| *it == 0x0A
	|| *it == 0x0D
	|| 0x7F < *it)
      return true;
  return false;
}

ostream &operator<<(ostream &out, const ClientStr &str) {
  return ClientStr::Send(out, str.str, NULL);
}

ostream &ClientStr::Send(ostream &out, ...) {
  va_list av;
  size_t total_size = 0;
  bool gotone = false;
  bool contains_unsafe = false;
  const String *str;

  va_start(av, out);
  while ((str = va_arg(av, const String *))) {
    gotone = true;
    total_size += str->size();
    contains_unsafe = contains_unsafe || contains_unsafe_chars(str);
  }
  va_end(av);

  if (! gotone)
    out << "NIL";
  else
    if (total_size > 1024 || contains_unsafe) {
      out << '{' << total_size << "}\r\n";
      va_start(av, out);
      while ((str = va_arg(av, const String *)))
	out << *str;
      va_end(av);
    } else {
      out << '\"';
      String::const_iterator it;
      va_start(av, out);
      while ((str = va_arg(av, const String *)))
	for (it = str->begin();
	     it != str->end();
	     ++it) {
	  if (*it == '\\' || *it == '\"')
	    out << '\\';
	  out << *it;
	}
      va_end(av);
      out << '\"';
    }
  return out;
}
