/* deletedsince.h -*- C++ -*- deletedsince command interface
 * Rob Earhart
 * $Id: deletedsince.h,v 1.4 1998/03/09 00:58:10 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_DELETEDSINCE_H
#define ACAP_DELETEDSINCE_H

#include "command.h"
#include "timestamp.h"
#include "dataset.h"
#include "response.h"
#include "interlock.h"

class DeletedSince : public Command {
public:
  DeletedSince(const String &_tag, Conn *_conn,
	       String _path, String _modtime)
  : Command(_tag, _conn), path(_path), modtime(_modtime) {
    if (path.size() < 1 || path[0] != '/')
      throw Response::BAD("DeletedSince dataset paths must begin with a '/'");
  }
  void Setup() {
    ts = Interlock::Instance()->CreateStamp();
  }
  ~DeletedSince();

private:
  typedef Dataset::names_t names_t;

  void Eval();
  Msg::Generator Name() const;

  String path;
  Ptr<Timestamp> ts;		// prevent data from being reused
				// while the command's in progress
  Timestamp::Stamp modtime;	// this is the actual time we're comparing deletedsince
				// info against...
};

#endif // ACAP_DELETEDSINCE_H
