/* key.h -*- C++ -*- thread key interface
 * Rob Earhart
 * $Id: key.h,v 1.2 1998/02/18 21:18:29 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_KEY_H
#define ACAP_KEY_H

#include <pthread.h>

// A key is a thread-global value that maps to some piece
// of thread-specific data - that is, the key can be shared
// by all threads, but each one will see a different value
// when dereferencing the key.

template<class V>
class Key {
public:
  Key(void (*destructor)(V *) = NULL) {
    pthread_key_create(&key, (void(*)(void*))destructor);
  }
  ~Key() {
    pthread_key_delete(key);
  }
  V *Get() { return (V *)pthread_getspecific(key); }
  void Set(V *value) { pthread_setspecific(key, value); }
private:
  pthread_key_t key;
};

// KeyValue objects are initialized with some per-thread key and some
// value, and they set the key to that value.  When they die, they
// unset it.  This is very cool for automatic cleanup as one's stack
// unwinds...
template<class V>
class KeyValue {
public:
  KeyValue(Key<V> &_key, V *value) : key(_key) {
    key.Set(value);
  }
  ~KeyValue() {
    key.Set(NULL);
  }
private:
  Key<V> &key;
};

#endif // ACAP_KEY_H
