/* octet.cc - octet comparator implementation
 * Rob Earhart
 * $Id: octet.cc,v 1.2 1997/12/14 23:18:16 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "compare.h"
#include "init.h"

static int OrdOctet(const String &first, const String &second) {
  return first.compare(second);
}

static int OrdOctetReverse(const String &first, const String &second) {
  return -OrdOctet(first, second);
}

static bool EqOctet(const String &a, const String &b) {
  return (a == b);
}

static bool MatchOctet(const String &a, const String &b, bool anchor) {
  // Someday, this ought to do boyer-moore matching or something... XXX
  if (anchor) {
    if (a.end() - a.begin() < b.end() - b.begin())
      return false;
    return equal(a.begin(), a.end(), b.begin());
  } else
    return search(a.begin(), a.end(), b.begin(), b.end()) != a.end();
}

static void SetupOctet() {
  Compare::Family *family = Compare::Instance()->FindFamily("i");
  family->Orderings()["i;octet"] = &OrdOctet;
  family->Orderings()["+i;octet"] = &OrdOctet;
  family->Orderings()["-i;octet"] = &OrdOctetReverse;
  family->Equalities()["i;octet"] = &EqOctet;
  family->Matches()["i;octet"] = &MatchOctet;
  family->Comparators().insert("i;octet");

  // Just for humor value, I'm also defining en-primary
  // as "octet".  I have no idea if this is correct...
  // (so where're these defined, anyway?  XXX)
  family = Compare::Instance()->FindFamily("en");
  family->Orderings()["en;primary"] = &OrdOctet;
  family->Orderings()["+en;primary"] = &OrdOctet;
  family->Orderings()["-en;primary"] = &OrdOctetReverse;
  family->Equalities()["en;primary"] = &EqOctet;
  family->Matches()["en;primary"] = &MatchOctet;
  family->Comparators().insert("en;primary");
}

static Init init(&SetupOctet);
