/* search.h -*- C++ -*- search command interface
 * Rob Earhart
 * $Id: search.h,v 1.5 1998/03/09 00:58:46 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_SEARCH_H
#define ACAP_SEARCH_H

#include <sys/types.h>
#include <slist.h>
#include "command.h"
#include "compare.h"
#include "timestamp.h"
#include "criteria.h"
#include "modifiers.h"
#include "searchable.h"

class Search : public Command {
public:
  Search(const String &tag, Conn *conn, const String &path);
  ~Search();
  void Setup();			// called after the command's state's
				// been completely initialized.  It acquires
				// a timestamp and requires that all previous
				// modification commands get threads, so it's
				// best to call this once the dispatch ordering
				// lock's been acquired.

  void SetCriteria(Ptr<Criteria> &_criteria) {
    criteria = _criteria;
  }

  Modifiers *GetModifiers() { return modifiers; }

  bool IsEnumeratedContext() { return is_enum_context; }
private:
  static Ptr<Criteria> subdataset_criteria;
  void Eval();
  Msg::Generator Name() const;
  bool is_enum_context;
  String path;
  Ptr<Criteria> criteria;
  Ptr<Modifiers> modifiers;
  Ptr<Timestamp> ts;
  NotifyContext *ncontext;
  P<Context> context;
  Ptr<Grab> context_grab;
  P<Searchable> searchable;
};

#endif // ACAP_SEARCH_H
