/* searchable.h -*- C++ -*- general searchable interface
 * Rob Earhart
 * $Id: searchable.h,v 1.2 1998/03/09 00:58:48 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_SEARCHABLE_H
#define ACAP_SEARCHABLE_H

#include <vector.h>
#include "ptr.h"
#include "str.h"
#include "modifiers.h"
#include "criteria.h"
#include "timestamp.h"
#include "refcount.h"
#include "dstring.h"

class EntryRef;
class DatasetGroup;
class NotifyContext;

// Refcounted is virtual so that both Searchable and PS can inherit from it,
// and Dataset can inherit from both.
class Searchable : virtual public Refcounted {
public:
  typedef vector<Ptr<EntryRef> > entries_t;
  struct Referer {
    virtual void operator()(const String &path,
			    const vector<DString> &urls) const = 0;
  };
  virtual void Search(entries_t &result,
		      DatasetGroup *group,
		      NotifyContext *context,
		      const String &path,
		      Modifiers *modifiers,
		      Criteria *criteria,
		      Referer *referer,
		      Timestamp::Stamp stamp) = 0;
};

#endif // ACAP_SEARCHABLE_H
