/* sort.cc - sort implementation
 * Rob Earhart
 * $Id: sort.cc,v 1.2 1998/03/09 00:58:49 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "sort.h"
#include "entryref.h"

int Sort::compare(const EntryRef *a, const EntryRef *b) const {
  // Unfortunately, this isn't cheap.
  for (sortlist_t::const_iterator it = sortlist.begin();
       it != sortlist.end();
       ++it) {
    Attribute attr_a = a->Lookup((*it).first);
    Attribute attr_b = b->Lookup((*it).first);
    if (! attr_a.IsString()) {
      if (! attr_b.IsString())
	return 0;		// both unuseable, so they're equal
      return 1;		// a undef, b def - b > a
    }
      if (! attr_b.IsString())
	return -1;		// a def, b undef - a > b
      int result = (*it).second(attr_a.StringValue(), attr_b.StringValue());
      if (result) return result;
      // Otherwise, we go on to the next sortlist entry...
  }
  // Otherwise, compare their relative paths, so that contexts will produce
  // reasonably minimal updates.
  return a->RelPath().compare(b->RelPath());
}
